/******************************************************************************
*
* project name:    TI-Mahjongg Solitaire
* file name:       board.c
* initial date:    06/11/2000
* authors:         thomas.nussbaumer@gmx.net
*                  marcos.lopez@wol.es
*
* description:     contains board handling and drawing functions
*
* $Id: board.c,v 1.2 2004/07/15 13:00:53 DEBROUX Lionel Exp $
*
******************************************************************************/

#define BOARD_SIZE_X    10   // don't change this
#define BOARD_SIZE_Y     6   // don't change this
#define BOARD_LAYERS     5   // maybe changed
#define SHUFFLE_ROUNDS 500   // number of random exchangements
#define NR_PIECES       32   // change this if you have more pieces

#define BOARD_FIELDS (BOARD_SIZE_X*BOARD_SIZE_Y*BOARD_LAYERS)

#include "setups.h"
#include "timah.h"


void InitCursorBuffer();
void PrintTime();


// offsets (mainly used for TI92plus)
//int offsetx = 0;
//int offsety = 0;

// the board and the back buffers
int*           board         = NULL;
unsigned char* backbuffer[2] = {NULL,NULL};

// buffers for cursor background backup
unsigned char* cursor_buf[2] = {NULL,NULL};

// how much pairs are left
unsigned int pairs_left = 0;

// variables to store a selected stone
int sel_layer = -1;
int sel_x     = -1;
int sel_y     = -1;


/*---------------------------------------------------------------------------*/
/* here comes the sprite data ...                                            */
/*---------------------------------------------------------------------------*/
#include "tiles.h"

unsigned int mask[16] = {
    0xc000,0x8000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,
    0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0001,0x0003
};

unsigned int sel_marker[16] = {
    0x3FFF,0x3FFF,0x3003,0x3003,0x3003,0x3003,0x3003,0x3003,
    0x3003,0x3003,0x3003,0x3003,0x3FFF,0x3FFF,0x0000,0x0000
};


unsigned int mask_clear[16] = {
    0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,
    0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000
};

unsigned int mask_helpview[16] = {
    ~0xc000,~0xc000,~0xc000,~0xc000,~0xc000,~0xc000,~0xc000,~0xc000,
    ~0xc000,~0xc000,~0xc000,~0xc000,~0xc000,~0xc000,~0xffff,~0xffff
};


/*===========================================================================*/
/* draws a single help field (position == 0 or == 1)                         */
/*===========================================================================*/
void UpdateHelpField(int type,int position) {
    int i;
    int x = 159-16+offsetx;
    int y = 27+offsety+position*(20);

    if (type == -1) {
        for (i=0;i<2;i++) {
            Sprite16(x,y,16,mask_clear, GetPlane(i),SPRT_AND);
        }
    }
    else {
        for (i=0;i<2;i++) {
            Sprite16(x,y,16,mask_clear, GetPlane(i),SPRT_AND);
            Sprite16(x,y,16,&tiles[i][type*16], GetPlane(i),SPRT_OR);
            Sprite16(x,y,16,mask_helpview, GetPlane(i),SPRT_AND);
        }
    }
}


/*===========================================================================*/
/* allocate space for some necessary globals                                 */
/*===========================================================================*/
int AllocateVariables() {
    if (!(board = malloc(BOARD_FIELDS*sizeof(int)))) return 0;
    if (!(backbuffer[0] = malloc(LCD_SIZE)))         return 0;
    if (!(backbuffer[1] = malloc(LCD_SIZE)))         return 0;
    if (!(cursor_buf[0] = malloc(250)))              return 0;
    if (!(cursor_buf[1] = malloc(250)))              return 0;

//    if (TI89) offsetx = offsety = 0;
//    else      offsetx = (240-160)/2,offsety = (128-100)/2;

    return 1;
}


/*===========================================================================*/
/* free allcated global again (CALL ONLY ONCE !!!)                           */
/*===========================================================================*/
void FreeVariables() {
    if (board)         free(board);
    if (backbuffer[0]) free(backbuffer[0]);
    if (backbuffer[1]) free(backbuffer[1]);
    if (cursor_buf[0]) free(cursor_buf[0]);
    if (cursor_buf[1]) free(cursor_buf[1]);
}


/*===========================================================================*/
/* initialize the board                                                      */
/*                                                                           */
/* returns 1 if everything is okay                                           */
/*===========================================================================*/
int InitBoard() {
    int            i;
    int           *tmpboard              = board;
    unsigned char *tmpsetup              = boardsetup[used_setup];
    int            count                 = 0;
    int            pieces[BOARD_FIELDS];

    sel_layer = sel_x = sel_y = -1;

    //----------------------------------------------------
    // count all pieces and check if it is a multiple of 2
    //----------------------------------------------------
    for (i=0;i<BOARD_FIELDS;i++) if (*tmpsetup++) count++;
    if ((count % 2)) return 0;

    pairs_left = count / 2;

    //----------------------------------------------------
    // fill pieces array with piece pairs
    //----------------------------------------------------
    for (i=0;i<count/4;i++) {
        pieces[i*4]   = i % NR_PIECES;
        pieces[i*4+1] = i % NR_PIECES;
        pieces[i*4+2] = i % NR_PIECES;
        pieces[i*4+3] = i % NR_PIECES;
    }
    for (i=0;i<count%4;i++) {
        pieces[4*(count/4)+i] = (4*(count/4)+i) % NR_PIECES;
    }

    //----------------------------------------------------
    // exchange random pieces with each other
    //----------------------------------------------------
    for (i=0;i<SHUFFLE_ROUNDS;i++) {
        int select1 = random(count);
        int select2 = random(count);
        int tmp     = pieces[select1];

        pieces[select1] = pieces[select2];
        pieces[select2] = tmp;
    }

    //----------------------------------------------------
    // fill up the board
    //----------------------------------------------------
    tmpsetup = boardsetup[used_setup];
    for (i=0;i<BOARD_FIELDS;i++) {
        if (*tmpsetup) *tmpboard = pieces[--count];
        else           *tmpboard = -1;
        tmpsetup++;
        tmpboard++;
    }

    return 1;
}


/*===========================================================================*/
/* draws a single field                                                      */
/*===========================================================================*/
void DrawField(int layer,int x,int y) {
    int i;

    int which = board[layer*BOARD_SIZE_X*BOARD_SIZE_Y+y*BOARD_SIZE_X+x];

    if (which != -1) {
        int offset = which * 16;
        int xx = offsetx+x*13+layer*2-BOARD_LAYERS*2+11;
        int yy = offsety+y*13-layer*2+BOARD_LAYERS*2-2;
        for (i=0;i<2;i++) {
            Sprite16(xx,yy,16,mask,             backbuffer[i],SPRT_AND);
            Sprite16(xx,yy,16,&tiles[i][offset],backbuffer[i],SPRT_OR);
        }

        if (layer == sel_layer && x == sel_x && y == sel_y) {
            for (i=0;i<2;i++) Sprite16(xx,yy,16,sel_marker,backbuffer[i],SPRT_OR);
        }
    }
}


/*===========================================================================*/
/* generates board display in backbuffer and copies it to display afterwards */
/*===========================================================================*/
void DrawBoard() {
    int x,y,layer;

    for (x=0;x<2;x++) memset(backbuffer[x],0,LCD_SIZE);

    for (layer=0;layer<BOARD_LAYERS;layer++) {
       for (y=0;y<BOARD_SIZE_Y;y++) {
           for (x=BOARD_SIZE_X-1;x>=0;x--) {
               DrawField(layer,x,y);
           }
       }
    }

    for (x=0;x<2;x++) memcpy(GetPlane(x),backbuffer[x],LCD_SIZE);
    InitCursorBuffer();
    PrintTime();
}


/*===========================================================================*/
/* useful macros to get and set stones                                       */
/*===========================================================================*/
#define GetStone(layer,x,y) board[BOARD_SIZE_X*BOARD_SIZE_Y*(layer)+BOARD_SIZE_X*(y)+(x)]
#define SetStone(layer,x,y,t) board[BOARD_SIZE_X*BOARD_SIZE_Y*(layer)+BOARD_SIZE_X*(y)+(x)] = (t)


/*===========================================================================*/
/* returns top layer at the given position or -1 if no stones are there      */
/*===========================================================================*/
int GetTopLayer(int x, int y) {
    int i;
    for (i=BOARD_LAYERS-1;i>=0;i--) {
        if (GetStone(i,x,y) != -1) return i;
    }

    return -1;
}


/*===========================================================================*/
/* unselects selected stone                                                  */
/*===========================================================================*/
void Unselect() {
    sel_layer = -1;
    sel_x     = -1;
    sel_y     = -1;
    DrawBoard();
}


/*===========================================================================*/
/* check if game is won (if there are still stones on the board)             */
/*===========================================================================*/
int CheckWin() {
    int i;
    int *b = board;

    for (i=0;i<BOARD_FIELDS;i++) {
        if (*b++ != -1) return 0;
    }

    return 1;
}


/*===========================================================================*/
/* check if there are still possible moves (return == 1)                     */
/*===========================================================================*/
int CheckPossibleMovesLeft() {
    int x,y;
    int valid_stones[BOARD_SIZE_X*BOARD_SIZE_Y];
    int top;
    int index = 0;
    int i;
    int stone;

    for (y=0;y<BOARD_SIZE_Y;y++) {
        for (x=0;x<BOARD_SIZE_X;x++) {

            top = GetTopLayer(x,y);
            if (top == -1) continue;

            //---------------------------------------------
            // check if the left/right free rule holds true
            //---------------------------------------------
            if (x != 0              && GetStone(top,x-1,y) != -1 &&
                x != BOARD_SIZE_X-1 && GetStone(top,x+1,y) != -1) continue;

            stone = GetStone(top,x,y);

            for (i=0;i<index;i++) {
                if (valid_stones[i] == stone) return 1;
            }
            valid_stones[index++] = stone;
        }
    }

    return 0;
}


/*===========================================================================*/
/* treatment of a select stone event                                         */
/*===========================================================================*/
int HandleSelect(int x,int y) {
    int top = GetTopLayer(x,y);
    if (top == -1) return 0;

    //---------------------------------------------
    // check if the left/right free rule holds true
    //---------------------------------------------
    if (x != 0              && GetStone(top,x-1,y) != -1 &&
        x != BOARD_SIZE_X-1 && GetStone(top,x+1,y) != -1) return 0;

    //-----------------------------
    // new selected stone ...
    //-----------------------------
    if (sel_layer == -1) {
        sel_layer = top;
        sel_x     = x;
        sel_y     = y;
        DrawBoard();
        UpdateHelpField(GetStone(top,x,y),0);
        return 0;
    }
    //-------------------------------------
    // deselect previous selected stone ...
    //-------------------------------------
    else if (x == sel_x && y == sel_y) {
        Unselect();
        UpdateHelpField(GetStone(top,x,y),0);
        UpdateHelpField(-1,1);
        return 0;
    }
    //-----------------------------------------
    // check types and remove them if equal ...
    //-----------------------------------------
    else {
        int type1 = GetStone(sel_layer,sel_x,sel_y);
        int type2 = GetStone(top,x,y);

        if (type1 == type2) {
            SetStone(sel_layer,sel_x,sel_y,-1);
            SetStone(top,x,y,-1);
            pairs_left--;
            Unselect();
            UpdateHelpField(GetStone(top,x,y),0);
            UpdateHelpField(-1,1);
        }
    }

    return CheckWin();
}

//=============================================================================
// Revision History
//=============================================================================
//
// $Log: board.c,v $
// Revision 1.2  2004/07/15 13:00:53  DEBROUX Lionel
// generic commit
//
// Revision 1.1  2000/11/14 19:14:00  Thomas Nussbaumer
// initial version
//
//
