/******************************************************************************
*
* project name:    TI-Mahjongg Solitaire
* file name:       cursor.c (initial code taken from TI-Minesweeper)
* initial date:    07/11/2000
* authors:         thomas.nussbaumer@gmx.net
*                  marcos.lopez@wol.es
*
* description:     cursor handling routines and cursor gfx data
*
*
* $Id: cursor.c,v 1.2 2004/07/15 13:01:52 DEBROUX Lionel Exp $
*
******************************************************************************/

#include "timah.h"

unsigned char pointer_plane[2][12] = {
    {0x80,0xc0,0xe0,0xb0,0xb8,0x9c,0x9e,0xb8,0xdc,0x9c,0x0e,0x0e},
    {0x80,0xc0,0xa0,0x90,0x88,0x84,0x8e,0xa8,0xd4,0x94,0x0a,0x0e}};
unsigned char mask_plane[2][12] = {
    {~0x00,~0x00,~0x40,~0x60,~0x70,~0x78,~0x70,~0x50,~0x08,~0x08,~0x04,~0x00},
    {~0x00,~0x00,~0x40,~0x60,~0x70,~0x78,~0x70,~0x50,~0x08,~0x08,~0x04,~0x00}};


SCR_RECT old_rect = {{255,255,255,255}};
extern SCR_RECT fullscreen;

/*===========================================================================*/
/* sets x0 to 255 to indicate that buffer contains no useful data            */
/*===========================================================================*/
void InitCursorBuffer() {
    old_rect.xy.x0 = 255;
}


/*===========================================================================*/
/* restores background of old cursor position                                */
/*===========================================================================*/
void RestoreOldCursorPos() {
    //-----------------------------------------------------------------
    // restore only if buffer contains data (x0==255 indicates no data)
    //-----------------------------------------------------------------
    if (old_rect.xy.x0 != 255) {
        int i;
        for (i=0;i<2;i++) {
            SetPlane(i);
            BitmapPut(old_rect.xy.x0,old_rect.xy.y0,cursor_buf[i],&fullscreen,A_REPLACE);
        }
    }
}


/*===========================================================================*/
/* backups background of cursor location (given x/y is the logical position) */
/*===========================================================================*/
void BackupCursorPos(int layer,int x,int y) {
    int i;
    //--------------------------------------------------------------
    // calculate physical x/y position and setup rectangle to backup
    //--------------------------------------------------------------
    x *= 13;
    x += offsetx+layer*2-BOARD_LAYERS*2+11+5+4;
    y *= 13;
    y += offsety-layer*2+BOARD_LAYERS*2-2+5+1+1;

    old_rect.xy.x0 = x-1;
    old_rect.xy.y0 = y-1;
    old_rect.xy.x1 = x+9;
    old_rect.xy.y1 = y+13;

    //----------------------------
    // get contents of both planes
    //----------------------------
    for (i=0;i<2;i++) {
        SetPlane(i);
        BitmapGet(&old_rect,cursor_buf[i]);
    }
}


/*===========================================================================*/
/* output cursor sprite at logical position (x/y)                            */
/*===========================================================================*/
void OutputCursor(int layer,int x,int y) {
    int i;
    //--------------------------------
    // calculate physical x/y position
    //--------------------------------
    x *= 13;
    x += offsetx+layer*2-BOARD_LAYERS*2+11+5+4;
    y *= 13;
    y += offsety-layer*2+BOARD_LAYERS*2-2+5+1;

    for (i=0;i<2;i++) {
        Sprite8(x,y,12,mask_plane[i],GetPlane(i),SPRT_AND);
        Sprite8(x,y,12,pointer_plane[i],GetPlane(i),SPRT_OR);
    }
}

//=============================================================================
// Revision History
//=============================================================================
//
// $Log: cursor.c,v $
// Revision 1.2  2004/07/15 13:01:52  DEBROUX Lionel
// generic commit
//
// Revision 1.1  2000/11/14 19:14:01  Thomas Nussbaumer
// initial version
//
//
